@@ -35,6 +35,7 @@
 #include "psycopg/microprotocols.h"
 #include "psycopg/microprotocols_proto.h"
 #include "pgversion.h"
+#include <stdlib.h>
 
 extern PyObject *pyPsycopgTzFixedOffsetTimezone;
 
@@ -497,9 +498,13 @@ psyco_curs_executemany(cursorObject *self, PyObject *args, PyObject *kwargs)
 {
     PyObject *operation = NULL, *vars = NULL;
     PyObject *v, *iter = NULL;
-
+    int rowcount = 0;
+    
     static char *kwlist[] = {"query", "vars_list", NULL};
 
+    /* reset rowcount to -1 to avoid setting it when an exception is raised */
+    self->rowcount = -1;
+    
     if (!PyArg_ParseTupleAndKeywords(args, kwargs, "OO", kwlist,
                                      &operation, &vars)) {
         return NULL;
@@ -525,11 +530,15 @@ psyco_curs_executemany(cursorObject *self, PyObject *args, PyObject *kwargs)
             return NULL;
         }
         else {
+            if (self->rowcount == -1)
+		rowcount = -1;
+            else if (rowcount >= 0)
+                rowcount += self->rowcount;
             Py_DECREF(v);
         }
     }
     Py_XDECREF(iter);
-    self->rowcount = -1;
+    self->rowcount = rowcount;
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -1113,7 +1122,7 @@ psyco_curs_scroll(cursorObject *self, PyObject *args, PyObject *kwargs)
 
 #ifdef PSYCOPG_EXTENSIONS
 
-#define COPY_BUFFER_SIZE 1024
+#define COPY_BUFFER_SIZE 8192
 
 static int _psyco_curs_copy_columns(PyObject *columns, char *columnlist)
 {
@@ -1187,21 +1196,23 @@ _psyco_curs_has_read_check(PyObject* o, void* var)
 static PyObject *
 psyco_curs_copy_from(cursorObject *self, PyObject *args, PyObject *kwargs)
 {
-    char query[DEFAULT_COPYBUFF];
+    char query_buffer[COPY_BUFFER_SIZE];
+    Py_ssize_t query_size;
+    char *query;
     const char *table_name;
     const char *sep = "\t", *null = NULL;
     Py_ssize_t bufsize = DEFAULT_COPYBUFF;
     PyObject *file, *columns = NULL, *res = NULL;
     char columnlist[DEFAULT_COPYBUFF];
+    char *quoted_delimiter;
 
-    static char *kwlist[] = {"file", "table", "sep", "null", "size",
-                             "columns", NULL};
+    static char *kwlist[] = {
+            "file", "table", "sep", "null", "size", "columns", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwargs,
         "O&s|ss" CONV_CODE_PY_SSIZE_T "O", kwlist,
         _psyco_curs_has_read_check, &file, &table_name, &sep, &null, &bufsize,
-        &columns)
-      )
+        &columns))
     {
         return NULL;
     }
@@ -1211,16 +1222,46 @@ psyco_curs_copy_from(cursorObject *self, PyObject *args, PyObject *kwargs)
 
     EXC_IF_CURS_CLOSED(self);
 
+    quoted_delimiter = psycopg_escape_string((PyObject*)self->conn, sep, 0, NULL, NULL);
+    if (quoted_delimiter == NULL) {
+        PyErr_NoMemory();
+        return NULL;
+    }
+    
+    query = query_buffer;
     if (null) {
-        PyOS_snprintf(query, DEFAULT_COPYBUFF-1,
-                      "COPY %s%s FROM stdin USING DELIMITERS '%s'"
-                      " WITH NULL AS '%s'", table_name, columnlist, sep, null);
+        char *quoted_null = psycopg_escape_string((PyObject*)self->conn, null, 0, NULL, NULL);
+        if (quoted_null == NULL) {
+            PyMem_Free(quoted_delimiter);
+            PyErr_NoMemory();
+            return NULL;
+        }
+        query_size = PyOS_snprintf(query, COPY_BUFFER_SIZE,
+            "COPY %s%s FROM stdin WITH DELIMITER AS %s NULL AS %s",
+            table_name, columnlist, quoted_delimiter, quoted_null);
+        if (query_size >= COPY_BUFFER_SIZE) {
+            /* Got truncated, allocate dynamically */
+            query = (char *)PyMem_Malloc((query_size + 1) * sizeof(char));
+            PyOS_snprintf(query, query_size + 1,
+                "COPY %s%s FROM stdin WITH DELIMITER AS %s NULL AS %s",
+                table_name, columnlist, quoted_delimiter, quoted_null);
+        }
+        PyMem_Free(quoted_null);
     }
     else {
-        PyOS_snprintf(query, DEFAULT_COPYBUFF-1,
-                      "COPY %s%s FROM stdin USING DELIMITERS '%s'",
-                      table_name, columnlist, sep);
-    }
+        query_size = PyOS_snprintf(query, COPY_BUFFER_SIZE,
+           "COPY %s%s FROM stdin WITH DELIMITER AS %s",
+           table_name, columnlist, quoted_delimiter);
+        if (query_size >= COPY_BUFFER_SIZE) {
+            /* Got truncated, allocate dynamically */
+            query = (char *)PyMem_Malloc((query_size + 1) * sizeof(char));
+            PyOS_snprintf(query, query_size + 1,
+                "COPY %s%s FROM stdin WITH DELIMITER AS %s",
+                table_name, columnlist, quoted_delimiter);
+        }
+    }    
+    PyMem_Free(quoted_delimiter);
+    
     Dprintf("psyco_curs_copy_from: query = %s", query);
 
     self->copysize = bufsize;
@@ -1231,7 +1272,10 @@ psyco_curs_copy_from(cursorObject *self, PyObject *args, PyObject *kwargs)
         Py_INCREF(Py_None);
     }
 
-    self->copyfile =NULL;
+    if (query && (query != query_buffer)) {
+        PyMem_Free(query);
+    }
+    self->copyfile = NULL;
 
     return res;
 }
@@ -1260,11 +1304,14 @@ _psyco_curs_has_write_check(PyObject* o, void* var)
 static PyObject *
 psyco_curs_copy_to(cursorObject *self, PyObject *args, PyObject *kwargs)
 {
-    char query[DEFAULT_COPYBUFF];
+    char *query = NULL;
+    char query_buffer[COPY_BUFFER_SIZE];
+    size_t query_size;
     char columnlist[DEFAULT_COPYBUFF];
     const char *table_name;
     const char *sep = "\t", *null = NULL;
     PyObject *file, *columns = NULL, *res = NULL;
+    char *quoted_delimiter;
 
     static char *kwlist[] = {"file", "table", "sep", "null", "columns", NULL};
 
@@ -1278,17 +1325,47 @@ psyco_curs_copy_to(cursorObject *self, PyObject *args, PyObject *kwargs)
         return NULL;
 
     EXC_IF_CURS_CLOSED(self);
-
+    quoted_delimiter = psycopg_escape_string((PyObject*)self->conn, sep, 0, NULL, NULL);
+    if (quoted_delimiter == NULL) {
+        PyErr_NoMemory();
+        return NULL;
+    }
+    
+    query = query_buffer;
     if (null) {
-        PyOS_snprintf(query, DEFAULT_COPYBUFF-1,
-                      "COPY %s%s TO stdout USING DELIMITERS '%s'"
-                      " WITH NULL AS '%s'", table_name, columnlist, sep, null);
+        char *quoted_null = psycopg_escape_string((PyObject*)self->conn, null, 0, NULL, NULL);
+        if (NULL == quoted_null) {
+            PyMem_Free(quoted_delimiter);
+            PyErr_NoMemory();
+            return NULL;
+        }
+        query_size = PyOS_snprintf(query, COPY_BUFFER_SIZE,
+            "COPY %s%s TO stdout WITH DELIMITER AS %s"
+            " NULL AS %s", table_name, columnlist, quoted_delimiter, quoted_null);
+        if (query_size >= COPY_BUFFER_SIZE) {
+            /* Got truncated, allocate dynamically */
+            query = (char *)PyMem_Malloc((query_size + 1) * sizeof(char));
+            PyOS_snprintf(query, query_size + 1,
+                "COPY %s%s TO stdout WITH DELIMITER AS %s"
+                " NULL AS %s", table_name, columnlist, quoted_delimiter, quoted_null);
+        }
+        PyMem_Free(quoted_null);
     }
     else {
-        PyOS_snprintf(query, DEFAULT_COPYBUFF-1,
-                      "COPY %s%s TO stdout USING DELIMITERS '%s'",
-                      table_name, columnlist, sep);
+        query_size = PyOS_snprintf(query, COPY_BUFFER_SIZE,
+            "COPY %s%s TO stdout WITH DELIMITER AS %s",
+            table_name, columnlist, quoted_delimiter);
+        if (query_size >= COPY_BUFFER_SIZE) {
+            /* Got truncated, allocate dynamically */
+            query = (char *)PyMem_Malloc((query_size + 1) * sizeof(char));
+            PyOS_snprintf(query, query_size + 1,
+                "COPY %s%s TO stdout WITH DELIMITER AS %s",
+                table_name, columnlist, quoted_delimiter);
+        }
     }
+    PyMem_Free(quoted_delimiter);
+    
+    Dprintf("psyco_curs_copy_to: query = %s", query);
 
     self->copysize = 0;
     self->copyfile = file;
@@ -1297,7 +1374,9 @@ psyco_curs_copy_to(cursorObject *self, PyObject *args, PyObject *kwargs)
         res = Py_None;
         Py_INCREF(Py_None);
     }
-
+    if (query && (query != query_buffer)) {
+        PyMem_Free(query);
+    }
     self->copyfile = NULL;
 
     return res;
@@ -1408,17 +1487,24 @@ psyco_curs_fileno(cursorObject *self, PyObject *args)
 static PyObject *
 psyco_curs_isready(cursorObject *self, PyObject *args)
 {
+    int res;
+    
     if (!PyArg_ParseTuple(args, "")) return NULL;
     EXC_IF_CURS_CLOSED(self);
 
     /* pq_is_busy does its own locking, we don't need anything special but if
        the cursor is ready we need to fetch the result and free the connection
-       for the next query. */
+       for the next query. if -1 is returned we raise an exception. */
 
-    if (pq_is_busy(self->conn)) {
+    res = pq_is_busy(self->conn);
+    
+    if (res == 1) {
         Py_INCREF(Py_False);
         return Py_False;
     }
+    else if (res == -1) {
+        return NULL;
+    }
     else {
         IFCLEARPGRES(self->pgres);
         Py_BEGIN_ALLOW_THREADS;
