@@ -27,42 +27,12 @@
 #include <libpq-fe.h>
 
 #include "psycopg/config.h"
+#include "psycopg/python.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-/* Python 2.5+ Py_ssize_t compatibility: */
-#ifndef PY_FORMAT_SIZE_T
-  #define PY_FORMAT_SIZE_T ""
-#endif
-
-/* FORMAT_CODE_SIZE_T is for plain size_t, not for Py_ssize_t: */
-#ifdef _MSC_VER
-  /* For MSVC: */
-  #define FORMAT_CODE_SIZE_T "%Iu"
-#else
-  /* C99 standard format code: */
-  #define FORMAT_CODE_SIZE_T "%zu"
-#endif
-/* FORMAT_CODE_PY_SSIZE_T is for Py_ssize_t: */
-#define FORMAT_CODE_PY_SSIZE_T "%" PY_FORMAT_SIZE_T "d"
-
-#if PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION >= 5
-  #define CONV_CODE_PY_SSIZE_T "n"
-#else
-  #define CONV_CODE_PY_SSIZE_T "i"
-
-  typedef int Py_ssize_t;
-  #define PY_SSIZE_T_MIN INT_MIN
-  #define PY_SSIZE_T_MAX INT_MAX
-
-  #define readbufferproc getreadbufferproc
-  #define writebufferproc getwritebufferproc
-  #define segcountproc getsegcountproc
-  #define charbufferproc getcharbufferproc
-#endif
-
 /* DBAPI compliance parameters */
 #define APILEVEL "2.0"
 #define THREADSAFETY 2
@@ -143,7 +113,8 @@ HIDDEN PyObject *psyco_GetDecimalType(void);
 HIDDEN void psyco_set_error(PyObject *exc, PyObject *curs,  const char *msg,
                             const char *pgerror, const char *pgcode);
 
-HIDDEN size_t qstring_escape(char *to, char *from, size_t len, PGconn *conn);
+HIDDEN char *psycopg_escape_string(PyObject *conn,
+              const char *from, Py_ssize_t len, char *to, Py_ssize_t *tolen);
 
 /* Exceptions docstrings */
 #define Error_doc \
@@ -174,7 +145,7 @@ HIDDEN size_t qstring_escape(char *to, char *from, size_t len, PGconn *conn);
 "Error related to problems with the processed data."
 
 #define NotSupportedError_doc \
-"A not supported datbase API was called."
+"A method or database API was used which is not supported by the database."
 
 #ifdef PSYCOPG_EXTENSIONS
 #define QueryCanceledError_doc \
